#include "config.h"

1 VERSIONINFO
BEGIN
  BLOCK "StringFileInfo"
  BEGIN
    BLOCK "040904E4"
    BEGIN
      VALUE "CompanyName", "Redshift Open Source Project"
      VALUE "FileDescription", "Redshift"
      VALUE "OriginalFilename", "redshift.exe"
      VALUE "ProductName", "Redshift"
      VALUE "ProductVersion", PACKAGE_VERSION
    END
  END
  BLOCK "VarFileInfo"
  BEGIN
    VALUE "Translation", 0x409, 1252
  END
END
